import Utils.IxLoadUtils as IxLoadUtils
import Utils.IxRestUtils as IxRestUtils
import inspect

####TEST CONFIG

kGatewayServer = "127.0.0.1"
kGatewayPort = 8443
kIxLoadVersion = "8.50.0.355"#TODO - to be changed by user
kChassisList = ['chassisIp']#TODO - to be changed by user
kOS="Windows"                             #TODO - to be changed by user depending on the remote OS Windows/Linux
location=inspect.getfile(inspect.currentframe())
WindowsPath=r"C:\\Path\\to\\Folder"  #TODO - to be changed by user if you run the script on Windows machine
                                     #This is the path where rxf file will be saved on the remote machine

kPortListPerCommunityCommunity =    {
                                        "Traffic1@Network1" : [(1,5,1)],
                                    }

kStatsToDisplayDict =   {
                            #format: { statSource : [stat name list] }
                            "DNSClient": ["DNS Simulated Users", "DNS Bytes"],
                        }

kCommunities = [
    # format: {option1: value1, option2: value2}
    {}  # default community with no options
]

kActivities = {
    # format: {communityName: [activityProtocolAndType1, activityProtocolAndType2]}
    'Traffic1@Network1': ['DNS Client'],
}

kDutType = {
                "type": "ExternalServer"
            }

kDutConfig = {
                "ipAddress": "10.10.10.10"
            }

################


#create a connection to the gateway
connection = IxRestUtils.getConnection(kGatewayServer, kGatewayPort)

sessionUrl = None

try:
    # create a session
    IxLoadUtils.log("Creating a new session...")
    sessionUrl = IxLoadUtils.createSession(connection, kIxLoadVersion)
    IxLoadUtils.log("Session created.")

    # create nettraffics (communities)
    IxLoadUtils.log('Creating communities...')
    IxLoadUtils.addCommunities(connection, sessionUrl, kCommunities)
    IxLoadUtils.log('Communities created.')

    # create activities
    IxLoadUtils.log('Creating activities...')
    IxLoadUtils.addActivities(connection, sessionUrl, kActivities)
    IxLoadUtils.log('Activities created.')

    IxLoadUtils.log('Adding DNS command to DNS client...')
    dnsCommandOptions = {'commandType': 'DnsQuery', "dnsServer": "DUT1"}
    IxLoadUtils.DnsUtils.addDnsCommand(connection, sessionUrl, 'Traffic1@Network1', 'DNSClient1', dnsCommandOptions)
    IxLoadUtils.log('Command added to DNS client.')

    IxLoadUtils.log("Adding DUT...")
    dutId = IxLoadUtils.addDUT(connection, sessionUrl, kDutType)
    IxLoadUtils.log("Finished adding DUT.")

    IxLoadUtils.log("Configuring DUT...")
    dutUrl = "%s/ixload/test/activeTest/dutList/%s" % (sessionUrl, dutId)
    IxLoadUtils.editDutConfig(connection, dutUrl, kDutConfig)
    IxLoadUtils.log("DUT configured.")

    IxLoadUtils.log("Clearing chassis list...")
    IxLoadUtils.clearChassisList(connection, sessionUrl)
    IxLoadUtils.log("Chassis list cleared.")

    IxLoadUtils.log("Adding chassis %s..." % (kChassisList))
    IxLoadUtils.addChassisList(connection, sessionUrl, kChassisList)
    IxLoadUtils.log("Chassis added.")

    IxLoadUtils.log("Assigning new ports...")
    IxLoadUtils.assignPorts(connection, sessionUrl, kPortListPerCommunityCommunity)
    IxLoadUtils.log("Ports assigned.")

    IxLoadUtils.log("Saving repository %s..." % (IxLoadUtils.getRxfName(kOS,location,WindowsPath)))
    IxLoadUtils.saveRxf(connection, sessionUrl, IxLoadUtils.getRxfName(kOS,location,WindowsPath))
    IxLoadUtils.log("Repository saved.")

    IxLoadUtils.log("Starting the test...")
    IxLoadUtils.runTest(connection, sessionUrl)
    IxLoadUtils.log("Test started.")

    IxLoadUtils.log("Polling values for stats %s..." % (kStatsToDisplayDict))
    IxLoadUtils.pollStats(connection, sessionUrl, kStatsToDisplayDict)

    IxLoadUtils.log("Test finished.")

    IxLoadUtils.log("Checking test status...")
    testRunError = IxLoadUtils.getTestRunError(connection, sessionUrl)
    if testRunError:
        IxLoadUtils.log("The test exited with the following error: %s" % testRunError)
    else:
        IxLoadUtils.log("The test completed successfully.")

    IxLoadUtils.log("Waiting for test to clean up and reach 'Unconfigured' state...")
    IxLoadUtils.waitForTestToReachUnconfiguredState(connection, sessionUrl)
    IxLoadUtils.log("Test is back in 'Unconfigured' state.")
finally:
    if sessionUrl is not None:
        IxLoadUtils.log("Closing IxLoad session...")
        IxLoadUtils.deleteSession(connection, sessionUrl)
        IxLoadUtils.log("IxLoad session closed.")
