import Utils.IxLoadUtils as IxLoadUtils
import Utils.IxRestUtils as IxRestUtils
import inspect
import os
import requests

####TEST CONFIG

kGatewayServer = "127.0.0.1"
kGatewayPort = 8443  # TODO - to be changed by user depending on whether HTTP redirect is used or not
# TODO - to be changed by user in order to use HTTP requests instead of HTTPS requests (the HTTP requests will be redirected as HTTPS requests)
kHttpRedirect = False
kIxLoadVersion = "8.40.0.277"  # TODO - to be changed by user
kRxfPath = r"C:\\Path\\to\\config.rxf"  # TODO - to be changed by user
kGatewaySharedFolder = 'C:\\ProgramData\\Ixia\\IxLoadGateway'  # TODO - to be changed by user depending on the gateway OS
kRxfRelativeUploadPath = 'uploads/%s' % os.path.split(kRxfPath)[1]  # TODO - to be changed by user
kRxfAbsoluteUploadPath = '/'.join([kGatewaySharedFolder, kRxfRelativeUploadPath])
kChassisList = ['chassisIpOrHostName']  # TODO - to be changed by user
kApiKey = ''                          # TODO - to be changed by user
kOS="Windows"                           #TODO - to be changed by user depending on the remote OS Windows/Linux
location=inspect.getfile(inspect.currentframe())
WindowsPath=r"C:\\Path\\to\\Folder"  #TODO - to be changed by user if you run the script on Windows machine
									 #This is the path where rxf file will be saved on the remote machine

kPortListPerCommunityCommunity =    {
                                        #  format: { community name : [ port list ] }
                                        "Traffic1@Network1" : [(1,5,1)],
                                        "Traffic2@Network2" : [(1,5,2)]
                                    }

kAnalyzerTupleList = [(0, '1.5.1'), (1, '1.5.2')] #TODO - to be changed by user, list of touples of community ids and port ids to apply Analyzer to
kCaptureFileRootPath = r"D:\\MyCaptures\\" #TODO - to by changed by user, root directory for capture files
kStatsToDisplayDict =   {
                            #format: { statSource : [stat name list] }
                            "HTTPClient": ["HTTP Simulated Users", "HTTP Concurrent Connections", "HTTP Requests Successful"],
                            "HTTPServer": ["HTTP Requests Received", "TCP Retries"]
                        }
################


#create a connection to the gateway
connection = IxRestUtils.getConnection(kGatewayServer, kGatewayPort, httpRedirect=kHttpRedirect)
connection.setApiKey(kApiKey)

sessionUrl = None
#create a session
try:
    IxLoadUtils.log("Creating a new session...")
    sessionUrl = IxLoadUtils.createSession(connection, kIxLoadVersion)
    IxLoadUtils.log("Session created.")

    # upload file to gateway server
    IxLoadUtils.log('Uploading file %s...' % kRxfPath)
    kResourcesUrl = IxLoadUtils.getResourcesUrl(connection)
    IxLoadUtils.uploadFile(connection, kResourcesUrl, kRxfPath, kRxfRelativeUploadPath)
    IxLoadUtils.log('Upload file finished.')

    # load a repository
    IxLoadUtils.log("Loading repository %s..." % kRxfAbsoluteUploadPath)
    IxLoadUtils.loadRepository(connection, sessionUrl, kRxfAbsoluteUploadPath)
    IxLoadUtils.log("Repository loaded.")

    IxLoadUtils.log("Clearing chassis list...")
    IxLoadUtils.clearChassisList(connection, sessionUrl)
    IxLoadUtils.log("Chassis list cleared.")

    IxLoadUtils.log("Adding chassis %s..." % (kChassisList))
    IxLoadUtils.addChassisList(connection, sessionUrl, kChassisList)
    IxLoadUtils.log("Chassis added.")

    IxLoadUtils.log("Assigning new ports...")
    IxLoadUtils.assignPorts(connection, sessionUrl, kPortListPerCommunityCommunity)
    IxLoadUtils.log("Ports assigned.")

    for analyzerTuple in kAnalyzerTupleList:
        IxLoadUtils.log("Applying analyzer to port [%s]..." % analyzerTuple[1])
        IxLoadUtils.enableAnalyzerOnPorts(connection, sessionUrl, analyzerTuple)
        IxLoadUtils.log("Finished applying analyzer to specified tuple")

    IxLoadUtils.log("Starting the test...")
    IxLoadUtils.runTest(connection, sessionUrl)
    IxLoadUtils.log("Test started.")

    IxLoadUtils.log("Waiting for capture data...")
    IxLoadUtils.waitForAllCaptureData(connection, sessionUrl)
    IxLoadUtils.log("Waiting done.")

    for analyzerTuple in kAnalyzerTupleList:
        communityID, portID = analyzerTuple
        IxLoadUtils.log("Retrieving capture for port %s..." % portID)
        captureFilePath = kCaptureFileRootPath + "Capture_%s_%s.cap" % (communityID, portID)
        IxLoadUtils.retrieveCaptureFileForPorts(connection, sessionUrl, analyzerTuple, captureFilePath)

    IxLoadUtils.log("Test finished.")

    IxLoadUtils.log("Checking test status...")
    testRunError = IxLoadUtils.getTestRunError(connection, sessionUrl)
    if testRunError:
        IxLoadUtils.log("The test exited with the following error: %s" % testRunError)
    else:
        IxLoadUtils.log("The test completed successfully.")

    IxLoadUtils.log("Waiting for test to clean up and reach 'Unconfigured' state...")
    IxLoadUtils.waitForTestToReachUnconfiguredState(connection, sessionUrl)
    IxLoadUtils.log("Test is back in 'Unconfigured' state.")
finally:
    if sessionUrl is not None:
        IxLoadUtils.log("Closing IxLoad session...")
        IxLoadUtils.deleteSession(connection, sessionUrl)
        IxLoadUtils.log("IxLoad session closed.")
